/**
 *  Copyright (C) 2009 Emerson Ribeiro de Mello, Fernando Fernandes Simão
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright (C) 2009 Emerson Ribeiro de Mello, Fernando Fernandes Simão
 *
 *  Este programa é software livre; você pode redistribuí-lo e/ou
 *  modificá-lo sob os termos da Licença Pública Geral GNU, conforme
 *  publicada pela Free Software Foundation; tanto a versão 2 da
 *  Licença como (a seu critério) qualquer versão mais nova.
 *
 *  Este programa é distribuído na expectativa de ser útil, mas SEM
 *  QUALQUER GARANTIA; sem mesmo a garantia implícita de
 *  COMERCIALIZAÇÃO ou de ADEQUAÇÃO A QUALQUER PROPÓSITO EM
 *  PARTICULAR. Consulte a Licença Pública Geral GNU para obter mais
 *  detalhes.
 *
 *  Você deve ter recebido uma cópia da Licença Pública Geral GNU
 *  junto com este programa; se não, escreva para a Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307, USA.
 */
package abstrata;

import java.awt.event.KeyEvent;
import java.util.ArrayList;
import javax.swing.JPanel;
import objetos.CanhaoDePlasma;
import objetos.Duke;
import objetos.Puffy;
import objetos.Tux;

/**
 * Classe Jogador contém o mínimo necessário para adicionar
 * um objeto da Tela e fazer com que este mova por toda sua
 * extensão.
 */
public abstract class Jogador extends Personagem {

    public ArrayList<CanhaoDePlasma> vetorCanhaoDePlasma; // vetor de tiros do personagem principal
    public ArrayList<CanhaoDePlasma> vetorCanhaoDePlasma1; // vetor de tiros do personagem principal
    public ArrayList<CanhaoDePlasma> vetorCanhaoDePlasma2; // vetor de tiros do personagem principal
    protected boolean invencibilidade; // para verificar se sofrerá colisões
    protected boolean ativarBombaAniquiladora;
    protected boolean ativarRevitalizador;
    private int velocidadeTiro = 2;
    public int velocidadePersonagem = 1;
    public int numeroDeTiros;
    public int numeroDeTiros1;
    public int numeroDeTiros2;
    public int potenciaCanhao;

    /**
     * * É necessário informar o nome da imagem que será carregada no jogador
     * @param nomeImagem caminho onde está localizada a imagem
     * @param jp referencia ao JPanel pai onde este objeto jogador está inserido
     * @param px posicao X inicial onde o jogador irá aparecer
     * @param py posicao Y inicial onde o jogador irá aparecer
     */
    public Jogador(String nomeImagem, JPanel jp, int px, int py) {
        super(nomeImagem, jp, px, py);
        vetorCanhaoDePlasma = new ArrayList<CanhaoDePlasma>();
        vetorCanhaoDePlasma1 = new ArrayList<CanhaoDePlasma>();
        vetorCanhaoDePlasma2 = new ArrayList<CanhaoDePlasma>();
        invencibilidade = false;
        ativarBombaAniquiladora = false;
        this.numeroDeTiros = 5;
        this.numeroDeTiros1 = 5;
        this.numeroDeTiros2 = 5;
        ativarRevitalizador = false;
        potenciaCanhao = 0;
    }

    /**
     * Para tratar a tecla que está pressionada. Enquanto a tecla de direção
     * estiver pressionada, o objeto será movido para a respectiva direção.
     * Neste método é alterado os atributos dirX e dirY que são usados pelo
     * método mover
     * @param evento tecla que foi pressionada
     */
    public void teclaPressionada(KeyEvent evento) {
        // obtendo o código da tecla que foi pressionado
        int tecla = evento.getKeyCode();

        switch (tecla) {
            case KeyEvent.VK_LEFT:
                dirX = -velocidadePersonagem; // movendo para a esquerda
                break;
            case KeyEvent.VK_RIGHT:
                dirX = velocidadePersonagem; // movendo para a direita
                break;
            case KeyEvent.VK_UP:
                dirY = -velocidadePersonagem; // movendo para cima
                break;
            case KeyEvent.VK_DOWN:
                dirY = velocidadePersonagem; // movendo para baixo
                break;
            case KeyEvent.VK_C:
                dispararCanhao();
                break;
            case KeyEvent.VK_R:
                if (this instanceof Duke) {
                    ((Duke) this).ativarRevitalizador(true);
                } else if (this instanceof Tux) {
                    ((Tux) this).ativarRevitalizador(true);
                }
                break;
            case KeyEvent.VK_E:
                if (this instanceof Duke) {
                    ((Duke) this).ativarEscudoDeForca(true);
                } else if (this instanceof Puffy) {
                    ((Puffy) this).ativarEscudoDeForca(true);
                }
                break;
            case KeyEvent.VK_B:
                if (this instanceof Puffy) {
                    ((Puffy) this).ativarBombaAniquiladora();
                } else if (this instanceof Tux) {
                    ((Tux) this).ativarBombaAniquiladora();
                }
                break;
        }
    }

    /**
     * Para de mover o objeto na direção da tecla que fora solta.
     * @param evento tecla que foi solta
     */
    public void teclaSolta(KeyEvent evento) {
        // obtendo o código da tecla que foi solta
        int tecla = evento.getKeyCode();

        switch (tecla) {
            case KeyEvent.VK_LEFT:
                dirX = 0; // para de mover para  esquerda
                break;
            case KeyEvent.VK_RIGHT:
                dirX = 0; // para de mover para  a direita
                break;
            case KeyEvent.VK_UP:
                dirY = 0; // para de mover para  cima
                break;
            case KeyEvent.VK_DOWN:
                dirY = 0; // para de mover para baixo
                break;
        }
    }

    /**
     * Aumenta a velocidade do tiro
     * @param v incrementa a velocidade do tiro do personagem principal
     */
    public void incrementarVelocidadeTiro(int v) {
        this.velocidadeTiro += v;
    }

    /**
     * Aumenta a velocidade de movimentação
     * @param vp incrementa velocidade de movimentação do personagem principal
     */
    public void incrementarVelocidadePersonagem(int vp) {
        this.velocidadePersonagem += vp;
    }

    /**
     * Aumenta a quantidade de disparos possíveis horizontais
     * @param d incrementa o número de disparos de canhao de plasma
     */
    public void incrementarNumeroDeDisparos(int d) {
        this.numeroDeTiros += d;
    }

    /**
     * Aumenta a quantidade de disparos possíveis diagonal superior
     * @param d incrementa o número de disparos de canhao de plasma
     */
    public void incrementarNumeroDeDisparosDiagonalSuperior(int ds) {
        this.numeroDeTiros1 += ds;
    }

    /**
     * Aumenta a quantidade de disparos possíveis diagonal inferior
     * @param d incrementa o número de disparos de canhao de plasma
     */
    public void incrementarNumeroDeDisparosDiagonalInferior(int di) {
        this.numeroDeTiros2 += di;
    }

    /**
     * Aumenta a potência do canhão de plasma
     * @param p aumenta a pontência do canhao de plasma
     */
    public void incrementarPotenciaCanhao(int p) {
        this.potenciaCanhao += p;
    }

    /**
     * Determina a velocidade do personagem
     * @param p define o valor da velocidade do personagem
     */
    public void setVelocidade(int v) {
        this.velocidadePersonagem = v;
    }

    /**
     * Determina a potência do canhão de plasma
     * @param p define o valor da pontência do canhao de plasma
     */
    public void setPotenciaCanhao(int p) {
        this.potenciaCanhao = p;
    }

    /**
     * adiciona um objeto CanhaoDePlasma no vetorCanhaoDePlasma
     * @param adiciona um objeto no vetorCanhaoDePlasma
     */
    public void dispararCanhao() {
        // limitando 5 disparos por vez
        if (vetorCanhaoDePlasma.size() < numeroDeTiros) {
            vetorCanhaoDePlasma.add(new CanhaoDePlasma("/imagens/laser.png", this.tela, x + this.getLargura(), y + this.getAltura() / 2 - 7, velocidadeTiro + 1));
        }
        if (potenciaCanhao >= 3) {
            if (vetorCanhaoDePlasma1.size() < numeroDeTiros1) {
                vetorCanhaoDePlasma1.add(new CanhaoDePlasma("/imagens/laser.png", this.tela, x + this.getLargura(), y + this.getAltura() / 2 - 7, velocidadeTiro + 1));
            }
        }
        if (vetorCanhaoDePlasma2.size() < numeroDeTiros2) {
            if (potenciaCanhao == 4) {
                vetorCanhaoDePlasma2.add(new CanhaoDePlasma("/imagens/laser.png", this.tela, x + this.getLargura(), y + this.getAltura() / 2 - 7, velocidadeTiro + 1));
            }
        }
    }

    /**
     * Retorna o conteúdo do vetor que armazena os disparos de Canhao de Plasma
     * @return 
     */
    public ArrayList<CanhaoDePlasma> obterCanhao() {
        return vetorCanhaoDePlasma;

    }

    /**
     * Retorna o conteúdo do vetor que armazena os disparos de Canhao de Plasma1
     * @return
     */
    public ArrayList<CanhaoDePlasma> obterCanhao1() {
        return vetorCanhaoDePlasma1;

    }

    /**
     * Retorna o conteúdo do vetor que armazena os disparos de Canhao de Plasma2
     * @return
     */
    public ArrayList<CanhaoDePlasma> obterCanhao2() {
        return vetorCanhaoDePlasma2;

    }

    /**
     * Ativa ou desativa a invencibilidade
     * @param i Se for true, ativa a invencibilidade, false, desativa
     */
    public void setInvencibilidade(boolean i) {
        this.invencibilidade = i;
    }

    /**
     * Retorna o valor atual da invencibilidade
     * Se for falso, personagem principal sofrerá danos
     * @return
     */
    public boolean obterInvencibilidade() {
        return invencibilidade;

    }

    /**
     * Ativa ou desativa a bomba aniquiladora
     * @param b Se for true, ativa a bomba, false, desativa
     */
    public void ativarBombaAniquiladora(boolean b) {
        this.ativarBombaAniquiladora = b;

    }

    /**
     * Retorna o estado da bomba aniquiladora - ativado/desativo
     * @return
     */
    public boolean obterAtivarBomba() {
        return ativarBombaAniquiladora;
    }

    public void ativarRevitalizador(boolean r) {
        this.ativarRevitalizador = r;
    }

    public boolean obterRevitalizador() {
        return ativarRevitalizador;
    }
}
