/**
 *  Copyright (C) 2009 Emerson Ribeiro de Mello, Fernando Fernandes Simão
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright (C) 2009 Emerson Ribeiro de Mello, Fernando Fernandes Simão
 *
 *  Este programa é software livre; você pode redistribuí-lo e/ou
 *  modificá-lo sob os termos da Licença Pública Geral GNU, conforme
 *  publicada pela Free Software Foundation; tanto a versão 2 da
 *  Licença como (a seu critério) qualquer versão mais nova.
 *
 *  Este programa é distribuído na expectativa de ser útil, mas SEM
 *  QUALQUER GARANTIA; sem mesmo a garantia implícita de
 *  COMERCIALIZAÇÃO ou de ADEQUAÇÃO A QUALQUER PROPÓSITO EM
 *  PARTICULAR. Consulte a Licença Pública Geral GNU para obter mais
 *  detalhes.
 *
 *  Você deve ter recebido uma cópia da Licença Pública Geral GNU
 *  junto com este programa; se não, escreva para a Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307, USA.
 */
package jogo;

import abstrata.Jogador;
import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.util.ArrayList;
import java.util.Random;
import javax.swing.JPanel;
import javax.swing.Timer;
import objetos.CanhaoDePlasma;
import objetos.Duke;
import objetos.Inimigo;
import objetos.Puffy;
import objetos.Tux;
import objetos.Chefao;
import objetos.PotenciaCanhaoDePlasma;
import objetos.Velocidade;

public class Tela extends JPanel implements ActionListener {

    private Timer timer; // para ficar desenhando a Tela2 de tempos em tempos
    private Random r; // só para colocar os jogadores em posições aleatórias na tela
    public boolean jogoAtivo; // para saber se o jogo está rodando
    public boolean inicial; // para saber se é um novo jogo
    private Jogo pai;
    private Jogador personagemPrincipal;
    private boolean ativarChefao;   // para ativar/desativar o chefão
    public ArrayList<Inimigo> inimigos; // vetor de inimigos
    public ArrayList<Chefao> chefoes;   // vetor de chefões
    public CanhaoDePlasma canhaodeplasma;
    private int quantidadeInimigos; // quantidade de inimigos a ser criada
    private int velocidadeInimigo = 1;  // velocidade inicial do inimigo
    private int tempoDoJogo;    // tempo usado para o inimigo disparar canhão
    private int tempoDoChefao;  // tempo usado para o chefão disparar canhão
    private float duracaoEscudoForca = 0;
    private int quantidadeDeTirosInimigo = 2; // quantidade de tiros inicial do inimigo
    private ArrayList<Velocidade> velocidade;
    private ArrayList<PotenciaCanhaoDePlasma> potenciacanhao;
    private boolean powerupVelocidade = false;
    private boolean powerupPotenciaCanhao = false;

    public Tela(Jogo p) {

        // Guardando um referencia ao JFrame PAI para pegar e modificar
        // alguns atributos que lá estão
        this.pai = p;

        tempoDoJogo = 0;
        tempoDoChefao = 0;
        this.r = new Random();

        // atributos que poderão ser modificados pela classe Principal
        this.jogoAtivo = true;
        this.inicial = false;

        // trata eventos das teclas pressionadas
        this.addKeyListener(new TratadorEventoTeclado());

        /* definindo algumas propriedades do JPanel */
        this.setFocusable(true); // assume foco

        this.setBackground(Color.black); // alterando a cor

        this.setDoubleBuffered(true); // tornando o redesenho mais rapido

        /* Irá atualizar a tela a cada 5 milisegundos */
        this.timer = new Timer(10, this);
        this.timer.start();
        this.inimigos = new ArrayList<Inimigo>();
        this.chefoes = new ArrayList<Chefao>();
        this.velocidade = new ArrayList<Velocidade>();
        this.potenciacanhao = new ArrayList<PotenciaCanhaoDePlasma>();
        this.ativarChefao = false;
    }

    /**
     * É chamado sempre que começar uma nova rodada
     */
    public void novaRodada() {
        // coordenadas do jogador principal
        int px = 10;
        int py = this.getHeight() / 2;
        /* criando jogador principal e posicionando na tela */
        if (this.pai.personagemEscolhido.equals("tux")) {
            personagemPrincipal = new Tux("/imagens/tux.png", this, px, py, this.pai);
        } else if (this.pai.personagemEscolhido.equals("duke")) {
            personagemPrincipal = new Duke("/imagens/duke.png", this, px, py, this.pai);
        } else if (this.pai.personagemEscolhido.equals("puffy")) {
            personagemPrincipal = new Puffy("/imagens/puffy.png", this, px, py, this.pai);
        }

        // definindo quantidade de inimigos aleatoriamente, minimo 1 maximo 5
        quantidadeInimigos = r.nextInt(5);
        if (quantidadeInimigos == 0) {
            quantidadeInimigos++;
        }
        // criando inimigos
        // olhar metodo
        criarInimigos(quantidadeInimigos);
        // duracao inicial do escudo de forca (invencibilidade)
        duracaoEscudoForca = 0;
    }

    /**
     * Irá desenhar os objetos na tela
     * Será chamado a cada 5 milisegundos, ou seja, a tela está em constante
     * atualização
     * @param g
     */
    @Override
    public void paint(Graphics g) {
        super.paint(g);
        Graphics2D g2d = (Graphics2D) g;

        if (jogoAtivo) { // o jogo está rodando!!!
            // Jogo acabou de ser carregado, mas ainda não foi iniciado um novo jogo
            // para criar os objetos somente 1 vez.
            if (inicial == true) {
                this.novaRodada(); //olhar dentro do metodo
                inicial = false;
            }
            if (personagemPrincipal != null) {
                // depuração
                pai.jLabelVelocidadeJogador.setText("" + personagemPrincipal.velocidadePersonagem);
                pai.jLabelDisparosJogador.setText("" + personagemPrincipal.numeroDeTiros);
                pai.jLabelPosiçãoJogador.setText("" + personagemPrincipal.getX() + "," + personagemPrincipal.getY());
                pai.jLabelPonteciaCanhao.setText("" + personagemPrincipal.potenciaCanhao);
                pai.jLabelVidaIJogador.setText("" + pai.jProgressBarVida.getValue());

                tempoDoJogo = (tempoDoJogo + 1) % 100;
                tempoDoChefao = (tempoDoChefao + 1) % 100;
                // desenhando o personagem principal
                g2d.drawImage(personagemPrincipal.getImagem(), personagemPrincipal.getX(), personagemPrincipal.getY(), this);
                // percorrendo o vetor de tiros e desenhando na tela
                ArrayList<CanhaoDePlasma> cp = personagemPrincipal.obterCanhao();
                ArrayList<CanhaoDePlasma> cp1 = personagemPrincipal.obterCanhao1();
                ArrayList<CanhaoDePlasma> cp2 = personagemPrincipal.obterCanhao2();
                for (CanhaoDePlasma c : cp) {
                    g2d.drawImage(c.getImagem(), c.getX(), c.getY(), this);
                }
                for (CanhaoDePlasma c1 : cp1) {
                    g2d.drawImage(c1.getImagem(), c1.getX(), c1.getY(), this);
                }
                for (CanhaoDePlasma c2 : cp2) {
                    g2d.drawImage(c2.getImagem(), c2.getX(), c2.getY(), this);
                }

                if (ativarChefao == true) {

                    ArrayList<CanhaoDePlasma> vetorRemoverTiroChefao = new ArrayList<CanhaoDePlasma>();
                    for (Chefao boss : chefoes) {
                        // tentando depurar
                        pai.jLabeAtivarChefao.setText("on");
                        pai.jLabelVidaChefao.setText("" + boss.vidaChefao);
                        pai.jLabelVelocidadeChefao.setText("" + boss.velocidadeChefao);
                        pai.jLabelPosiçãoChefao.setText("" + boss.getX() + "," + boss.getY());

                        g2d.drawImage(boss.getImagem(), boss.getX(), boss.getY(), this);
                        if ((boss.canhaoChefao1.size() < 5) && (tempoDoJogo == r.nextInt(100))) {
                            boss.canhaoChefao1.add(new CanhaoDePlasma("/imagens/plasmac.png", this, boss.getX() - 10, boss.getY() + 50, boss.velocidadeChefao + 2));
                        }
                        if ((boss.canhaoChefao2.size() < 5) && (tempoDoJogo == r.nextInt(100))) {
                            boss.canhaoChefao2.add(new CanhaoDePlasma("/imagens/plasmac.png", this, boss.getX(), boss.getY() + 100, boss.velocidadeChefao + 2));
                        }
                        if ((boss.canhaoChefao3.size() < 5) && (tempoDoJogo == r.nextInt(100))) {
                            boss.canhaoChefao3.add(new CanhaoDePlasma("/imagens/plasmac.png", this, boss.getX(), boss.getY(), boss.velocidadeChefao + 2));
                        }
                        for (CanhaoDePlasma cC : boss.canhaoChefao1) {
                            g2d.drawImage(cC.getImagem(), cC.getX(), cC.getY(), this);
                            // dispara se está dentro dos limites da tela
                            if (cC.visibilidadeTiro() == true) {
                                cC.dispararCanhaoDePlasma(false);
                            } else {
                                vetorRemoverTiroChefao.add(cC);
                            }
                        }
                        for (CanhaoDePlasma cC : boss.canhaoChefao2) {
                            g2d.drawImage(cC.getImagem(), cC.getX(), cC.getY(), this);
                            // dispara se está dentro dos limites da tela
                            if (cC.visibilidadeTiro() == true) {
                                cC.dispararCanhaoDePlasma(false);
                            } else {
                                vetorRemoverTiroChefao.add(cC);
                            }
                        }
                        for (CanhaoDePlasma cC : boss.canhaoChefao3) {
                            g2d.drawImage(cC.getImagem(), cC.getX(), cC.getY(), this);
                            // dispara se está dentro dos limites da tela
                            if (cC.visibilidadeTiro() == true) {
                                cC.dispararCanhaoDePlasma(false);
                            } else {
                                vetorRemoverTiroChefao.add(cC);
                            }
                        }
                        boss.movimentarChefao();
                        for (CanhaoDePlasma cC : vetorRemoverTiroChefao) {
                            boss.canhaoChefao1.remove(cC);
                        }
                        for (CanhaoDePlasma cC : vetorRemoverTiroChefao) {
                            boss.canhaoChefao2.remove(cC);
                        }
                        for (CanhaoDePlasma cC : vetorRemoverTiroChefao) {
                            boss.canhaoChefao3.remove(cC);
                        }
                    }
                }
                // percorrendo vetor de inimigos e desenhando na tela
                int indiceinimigos = 0;
                for (Inimigo ini : inimigos) {
                    ArrayList<CanhaoDePlasma> vetorRemoverTiroInimigo = new ArrayList<CanhaoDePlasma>();
                    // desenhando o inimigo
                    g2d.drawImage(ini.getImagem(), ini.getX(), ini.getY(), this);
                    // tentando depurar
                    pai.jLabelQtdInimigosTela.setText("" + inimigos.size());
                    if (indiceinimigos == 0) {
                        pai.jLabelPosiçãoVetor.setText("" + (indiceinimigos + 1));
                        pai.jLabelVidaInimigo.setText("" + inimigos.get(indiceinimigos).vidaInimigo);
                        pai.jLabelVelocidade.setText("" + inimigos.get(indiceinimigos).velocidadeInimigo);
                        pai.jLabelPosiçãoInimigoXY.setText("" + inimigos.get(indiceinimigos).getX() + "," + inimigos.get(indiceinimigos).getY());
                    }
                    if (indiceinimigos == 1) {
                        pai.jLabelPosiçãoVetor1.setText("" + (indiceinimigos + 1));
                        pai.jLabelVidaInimigo1.setText("" + inimigos.get(indiceinimigos).vidaInimigo);
                        pai.jLabelVelocidade1.setText("" + inimigos.get(indiceinimigos).velocidadeInimigo);
                        pai.jLabelPosiçãoInimigoXY1.setText("" + inimigos.get(indiceinimigos).getX() + "," + inimigos.get(indiceinimigos).getY());
                    }
                    if (indiceinimigos == 2) {
                        pai.jLabelPosiçãoVetor2.setText("" + (indiceinimigos + 1));
                        pai.jLabelVidaInimigo2.setText("" + inimigos.get(indiceinimigos).vidaInimigo);
                        pai.jLabelVelocidade2.setText("" + inimigos.get(indiceinimigos).velocidadeInimigo);
                        pai.jLabelPosiçãoInimigoXY2.setText("" + inimigos.get(indiceinimigos).getX() + "," + inimigos.get(indiceinimigos).getY());
                    }
                    if (indiceinimigos == 3) {
                        pai.jLabelPosiçãoVetor3.setText("" + (indiceinimigos + 1));
                        pai.jLabelVidaInimigo3.setText("" + inimigos.get(indiceinimigos).vidaInimigo);
                        pai.jLabelVelocidade3.setText("" + inimigos.get(indiceinimigos).velocidadeInimigo);
                        pai.jLabelPosiçãoInimigoXY3.setText("" + inimigos.get(indiceinimigos).getX() + "," + inimigos.get(indiceinimigos).getY());
                    }
                    if (indiceinimigos == 4) {
                        pai.jLabelPosiçãoVetor4.setText("" + (indiceinimigos + 1));
                        pai.jLabelVidaInimigo4.setText("" + inimigos.get(indiceinimigos).vidaInimigo);
                        pai.jLabelVelocidade4.setText("" + inimigos.get(indiceinimigos).velocidadeInimigo);
                        pai.jLabelPosiçãoInimigoXY4.setText("" + inimigos.get(indiceinimigos).getX() + "," + inimigos.get(indiceinimigos).getY());
                    }
                    indiceinimigos++;


                    // cada inimigo poderá disparar duas vezes por tela ou antes de colisão com jogador
                    if ((ini.canhaoInimigo.size() < quantidadeDeTirosInimigo) && (tempoDoJogo == r.nextInt(100))) {
                        ini.canhaoInimigo.add(new CanhaoDePlasma("/imagens/laseri.png", this, ini.getX() - ini.getLargura(), ini.getY() + ini.getAltura() / 2 - 7, ini.velocidadeInimigo + 2));
                    }
                    // desenhando os disparos do inimigos e movimentando na tela
                    for (CanhaoDePlasma cI : ini.canhaoInimigo) {
                        g2d.drawImage(cI.getImagem(), cI.getX(), cI.getY(), this);
                        // dispara se está dentro dos limites da tela
                        if (cI.visibilidadeTiro() == true) {
                            cI.dispararCanhaoDePlasma(false);
                            // remove após limites da tela
                            } else {
                            vetorRemoverTiroInimigo.add(cI);
                        }
                    }
                    // movimentando inimigos
                    ini.movimentarInimigo();
                    for (CanhaoDePlasma cI : vetorRemoverTiroInimigo) {
                        ini.canhaoInimigo.remove(cI);
                    }
                }
                if (powerupVelocidade == true) {
                    for (Velocidade v : velocidade) {
                        g2d.drawImage(v.getImagem(), v.getX(), v.getY(), this);
                        v.moverVelocidade();
                    }
                }
                if (powerupPotenciaCanhao == true) {
                    for (PotenciaCanhaoDePlasma p : potenciacanhao) {
                        g2d.drawImage(p.getImagem(), p.getX(), p.getY(), this);
                        p.moverPotenciaCanhaoDePlasma();
                    }
                }

                // verificando se o escudo de força está ativado
                if (personagemPrincipal.obterInvencibilidade() == true) {
                    pai.jLabelEscudoDeForça.setText("on");
                    // incrementa o tempo de duracao, de 0 a 999, atualizado
                    // a cada 5ms (repaint)
                    duracaoEscudoForca = (duracaoEscudoForca + 1) % 500;
                    pai.jLabelDuraçãoEscudoForça.setText("" + duracaoEscudoForca / 100);
                    pai.jProgressBarEscudoDeForça.setValue((int) duracaoEscudoForca);
                    if (duracaoEscudoForca == 499) {
                        duracaoEscudoForca = 0;
                        pai.jProgressBarEscudoDeForça.setValue((int) duracaoEscudoForca);
                        pai.jLabelEscudoDeForça.setText("off");
                        pai.jLabelDuraçãoEscudoForça.setText("" + duracaoEscudoForca);

                        personagemPrincipal.setInvencibilidade(false);
                        if (personagemPrincipal instanceof Duke) {
                            personagemPrincipal.alterarImagem("/imagens/duke.png");
                        }
                        if (personagemPrincipal instanceof Puffy) {
                            personagemPrincipal.alterarImagem("/imagens/puffy.png");
                        }
                    }
                }
                // verficando se a bomba aniquiladora foi ativada
                if (personagemPrincipal.obterAtivarBomba() == true) {
                    ArrayList<Inimigo> aniquilarInimigo = new ArrayList<Inimigo>();
                    ArrayList<CanhaoDePlasma> aniquilarCanhaoInimigo = new ArrayList<CanhaoDePlasma>();
                    for (Inimigo aI : inimigos) {
                        // adicionando inimigo no vetor para ser removido
                        aniquilarInimigo.add(aI);
                        // aumentando um ponto para cada inimigo morto
                        int pontos = new Integer(pai.jLabelPontos.getText()).intValue();
                        pontos++;
                        pai.jProgressBarPoder.setValue(pai.jProgressBarPoder.getValue() + 5);
                        pai.jLabelPontos.setText("" + pontos);
                        // se a pontuação for múltiplo de 6, a velocidade do inimigo aumenta
                        if ((pontos > 0) && (pontos % 6 == 0)) {
                            velocidadeInimigo++;
                            quantidadeDeTirosInimigo++;
                        }
                        // se a pontuação for múltiplo de 50, o chefão é criado
                        if ((pontos > 0) && (pontos % 50 == 0) && (ativarChefao = !true)) {
                            ativarChefao = true;
                            chefoes.add(new Chefao("/imagens/chefao.png", this, pai.jPanelTela.getWidth() - 195, pai.jPanelTela.getHeight() / 4, velocidadeInimigo));
                            pai.jProgressBarVidaChefao.setValue(500);
                        }
                        // se a pontuação for múltiplo de 10, um power up é criado
                        if (pontos % 6 == 0) {
                            boolean powerup = r.nextBoolean();
                            if (powerup == true) {
                                if (pai.jProgressBarVelocidade.getValue() < 100) {
                                    velocidade.add(new Velocidade("/imagens/velocidade.png", this, aI.getX(), aI.getY()));
                                    powerupVelocidade = true;
                                } else if (pai.jProgressBarPotenciaCanhao.getValue() < 100) {
                                    potenciacanhao.add(new PotenciaCanhaoDePlasma("/imagens/potencia.png", this, aI.getX(), aI.getY()));
                                    powerupPotenciaCanhao = true;
                                }
                            }
                            if (powerup == false) {
                                if (pai.jProgressBarPotenciaCanhao.getValue() < 100) {
                                    potenciacanhao.add(new PotenciaCanhaoDePlasma("/imagens/potencia.png", this, aI.getX(), aI.getY()));
                                    powerupPotenciaCanhao = true;
                                } else if (pai.jProgressBarVelocidade.getValue() < 100) {
                                    velocidade.add(new Velocidade("/imagens/velocidade.png", this, aI.getX(), aI.getY()));
                                    powerupVelocidade = true;
                                }
                            }
                            for (CanhaoDePlasma aCI : aI.canhaoInimigo) {
                                // adicionando disparo do inimigo para ser removido
                                aniquilarCanhaoInimigo.add(aCI);
                            }
                        }
                    }
                    // removando inimigos e disparos inimigos da tela
                    for (Inimigo aI2 : aniquilarInimigo) {
                        for (CanhaoDePlasma aCI2 : aniquilarCanhaoInimigo) {
                            aI2.canhaoInimigo.remove(aCI2);
                        }
                        inimigos.remove(aI2);
                    }
                    // desativando a bomba aniquiladora
                    personagemPrincipal.ativarBombaAniquiladora(false);
                    // criando novos inimigos
                    quantidadeInimigos = r.nextInt(5) + 1;
                    criarInimigos(quantidadeInimigos);
                }
                // para garantir que a atualização da tela seja síncrona
                Toolkit.getDefaultToolkit().sync();
                // liberando os recursos do sistema usados para gerar os gráficos acima
                g.dispose();
            }
        } else if (pai.jProgressBarVida.getValue() == 0) { // fim de jogo, imprima a mensagem

            // ativando o antialiasing
            g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
            // modificando as características fonte
            Font small = new Font("Helvetica", Font.BOLD, 14);
            FontMetrics metr = this.getFontMetrics(small);
            g2d.setColor(Color.black);
            g2d.setFont(small);

            String msg = "Fim de Jogo!";

            g2d.drawString(msg, (this.getWidth() - metr.stringWidth(msg)) / 2,
                    this.getHeight() / 2);
            prepararNovoJogo();
            pai.jMenuItemNovo.setEnabled(true);
            pai.jMenuItemCancelarJogo.setEnabled(false);
            // gravando recorde
            int recorde = new Integer(pai.jLabelRecordeMax.getText()).intValue();
            int pontos = new Integer(pai.jLabelPontos.getText()).intValue();
            if (pontos > recorde) {
                pai.jLabelRecordeMax.setText("" + pontos);
            }
        }
    }

    /**
     * Se o jogo acabar, reseta os valores atuais para iniciais
     */
    public void prepararNovoJogo() {
        // velocidade inicial do inimigo
        // desativando chefão
        velocidadeInimigo = 1;
        ativarChefao = false;
        quantidadeDeTirosInimigo = 2;
        personagemPrincipal.setPotenciaCanhao(0);
        pai.jProgressBarVelocidade.setValue(0);
        pai.jProgressBarPotenciaCanhao.setValue(0);
        pai.jProgressBarVidaChefao.setValue(0);
        // removendo inimigos e tiros
        ArrayList<Inimigo> aniquilarInimigo = new ArrayList<Inimigo>();
        ArrayList<CanhaoDePlasma> aniquilarCanhaoInimigo = new ArrayList<CanhaoDePlasma>();
        for (Inimigo aI : inimigos) {
            aniquilarInimigo.add(aI);
            for (CanhaoDePlasma aCI : aI.canhaoInimigo) {
                aniquilarCanhaoInimigo.add(aCI);
            }
        }
        for (Inimigo aI2 : aniquilarInimigo) {
            for (CanhaoDePlasma aCI2 : aniquilarCanhaoInimigo) {
                aI2.canhaoInimigo.remove(aCI2);
            }
            inimigos.remove(aI2);
        }
        // removendo chefao e tiros
        ArrayList<Chefao> aniquilarChefao = new ArrayList<Chefao>();
        ArrayList<CanhaoDePlasma> aniquilarCanhaoChefao1 = new ArrayList<CanhaoDePlasma>();
        ArrayList<CanhaoDePlasma> aniquilarCanhaoChefao2 = new ArrayList<CanhaoDePlasma>();
        ArrayList<CanhaoDePlasma> aniquilarCanhaoChefao3 = new ArrayList<CanhaoDePlasma>();
        for (Chefao aC : chefoes) {
            aniquilarChefao.add(aC);
            for (CanhaoDePlasma aCC : aC.canhaoChefao1) {
                aniquilarCanhaoChefao1.add(aCC);
            }
            for (CanhaoDePlasma aCC : aC.canhaoChefao2) {
                aniquilarCanhaoChefao2.add(aCC);
            }
            for (CanhaoDePlasma aCC : aC.canhaoChefao3) {
                aniquilarCanhaoChefao3.add(aCC);
            }
        }
        for (Chefao aC2 : aniquilarChefao) {
            for (CanhaoDePlasma aCC2 : aniquilarCanhaoChefao1) {
                aC2.canhaoChefao1.remove(aCC2);
            }
            chefoes.remove(aC2);
        }
        for (Chefao aC3 : aniquilarChefao) {
            for (CanhaoDePlasma aCC3 : aniquilarCanhaoChefao2) {
                aC3.canhaoChefao1.remove(aCC3);
            }
            chefoes.remove(aC3);
        }
        for (Chefao aC4 : aniquilarChefao) {
            for (CanhaoDePlasma aCC4 : aniquilarCanhaoChefao3) {
                aC4.canhaoChefao1.remove(aCC4);
            }
            chefoes.remove(aC4);
        }
        ArrayList<Velocidade> removerVelocidade = new ArrayList<Velocidade>();
        ArrayList<PotenciaCanhaoDePlasma> removerPotencia = new ArrayList<PotenciaCanhaoDePlasma>();
        for (Velocidade v : velocidade) {
            removerVelocidade.add(v);
        }
        for (PotenciaCanhaoDePlasma p : potenciacanhao) {
            removerPotencia.add(p);
        }
        for (Velocidade v : removerVelocidade) {
            velocidade.remove(v);
        }
        for (PotenciaCanhaoDePlasma p : removerPotencia) {
            potenciacanhao.remove(p);
        }
    }

    /**
     * Irá tratar todos os eventos que ocorrerem no JPanel.
     * O pressionar de uma tecla é um evento a ser tratado.
     *
     * @param e evento que ocorreu na Tela2
     */
    public void actionPerformed(ActionEvent e) {
        // para movimentar o jogador pela tela
        // somente depois que o objeto for criado, ou seja, depois de um novo jogo
        if (personagemPrincipal != null) {
            // movimentando jogador
            personagemPrincipal.mover();
            // movimentando os disparos do jogador principal
            ArrayList<CanhaoDePlasma> cp = personagemPrincipal.obterCanhao();
            ArrayList<CanhaoDePlasma> cp1 = personagemPrincipal.obterCanhao1();
            ArrayList<CanhaoDePlasma> cp2 = personagemPrincipal.obterCanhao2();
            ArrayList<CanhaoDePlasma> tiroDescartado = new ArrayList<CanhaoDePlasma>();
            ArrayList<CanhaoDePlasma> tiroDescartado1 = new ArrayList<CanhaoDePlasma>();
            ArrayList<CanhaoDePlasma> tiroDescartado2 = new ArrayList<CanhaoDePlasma>();
            int indice = 0;
            for (CanhaoDePlasma c : cp) {
                // se ultrapassou limite de largura da tela, é descartado
                if (c.visibilidadeTiro() == true) {
                    c.dispararCanhaoDePlasma(true);
                    // tentando depurar
                    if (indice == 0) {
                        pai.jLabelDanoCanhao1.setText("" + c.obterDanoCanhaoDePlasma());
                    }
                    if (indice == 1) {
                        pai.jLabelDanoCanhao2.setText("" + c.obterDanoCanhaoDePlasma());
                    }
                    if (indice == 2) {
                        pai.jLabelDanoCanhao3.setText("" + c.obterDanoCanhaoDePlasma());
                    }
                    if (indice == 3) {
                        pai.jLabelDanoCanhao4.setText("" + c.obterDanoCanhaoDePlasma());
                    }
                    if (indice == 4) {
                        pai.jLabelDanoCanhao5.setText("" + c.obterDanoCanhaoDePlasma());
                    }
                    if (indice == 5) {
                        pai.jLabelDanoCanhao6.setText("" + c.obterDanoCanhaoDePlasma());
                    }
                    if (indice == 6) {
                        pai.jLabelDanoCanhao7.setText("" + c.obterDanoCanhaoDePlasma());
                    }
                } else {
                    tiroDescartado.add(c);
                }
                indice++;
            }
            int indice1 = 0;
            for (CanhaoDePlasma c1 : cp1) {
                // se ultrapassou limite de largura da tela, é descartado
                if (c1.visibilidadeTiro() == true) {
                    c1.dispararCanhaoDePlasmaDiagonal(true);
                    // tentando depurar
                    if (indice1 == 0) {
                        pai.jLabelDanoCanhaoDS1.setText("" + c1.obterDanoCanhaoDePlasma());
                    }
                    if (indice1 == 1) {
                        pai.jLabelDanoCanhaoDS2.setText("" + c1.obterDanoCanhaoDePlasma());
                    }
                    if (indice1 == 2) {
                        pai.jLabelDanoCanhaoDS3.setText("" + c1.obterDanoCanhaoDePlasma());
                    }
                    if (indice1 == 3) {
                        pai.jLabelDanoCanhaoDS4.setText("" + c1.obterDanoCanhaoDePlasma());
                    }
                    if (indice1 == 4) {
                        pai.jLabelDanoCanhaoDS5.setText("" + c1.obterDanoCanhaoDePlasma());
                    }
                    if (indice1 == 5) {
                        pai.jLabelDanoCanhaoDS6.setText("" + c1.obterDanoCanhaoDePlasma());
                    }
                    if (indice1 == 6) {
                        pai.jLabelDanoCanhaoDS7.setText("" + c1.obterDanoCanhaoDePlasma());
                    }
                } else {
                    tiroDescartado1.add(c1);
                }
                indice1++;
            }
            int indice2 = 0;
            for (CanhaoDePlasma c2 : cp2) {
                // se ultrapassou limite de largura da tela, é descartado
                if (c2.visibilidadeTiro() == true) {
                    c2.dispararCanhaoDePlasmaDiagonal(false);
                    // tentando depurar
                    if (indice2 == 0) {
                        pai.jLabelDanoCanhaoDI1.setText("" + c2.obterDanoCanhaoDePlasma());
                    }
                    if (indice2 == 1) {
                        pai.jLabelDanoCanhaoDI2.setText("" + c2.obterDanoCanhaoDePlasma());
                    }
                    if (indice2 == 2) {
                        pai.jLabelDanoCanhaoDI3.setText("" + c2.obterDanoCanhaoDePlasma());
                    }
                    if (indice2 == 3) {
                        pai.jLabelDanoCanhaoDI4.setText("" + c2.obterDanoCanhaoDePlasma());
                    }
                    if (indice2 == 4) {
                        pai.jLabelDanoCanhaoDI5.setText("" + c2.obterDanoCanhaoDePlasma());
                    }
                    if (indice2 == 5) {
                        pai.jLabelDanoCanhaoDI6.setText("" + c2.obterDanoCanhaoDePlasma());
                    }
                    if (indice2 == 6) {
                        pai.jLabelDanoCanhaoDI7.setText("" + c2.obterDanoCanhaoDePlasma());
                    }
                } else {
                    tiroDescartado2.add(c2);
                }
                indice2++;
            }
            // removendo tiros fora dos limites da tela
            for (CanhaoDePlasma c : tiroDescartado) {
                cp.remove(c);
            }
            for (CanhaoDePlasma c1 : tiroDescartado1) {
                cp1.remove(c1);
            }
            for (CanhaoDePlasma c2 : tiroDescartado2) {
                cp2.remove(c2);
            }
            // verficando colisões
            this.verificaColisoes();
            // repintando a tela
            repaint();
        }
    }

    /**
     * Cria os inimigos iniciais e os sucessivos.
     * @param quantidade quantos inimigos serão criados após a morte de um
     */
    public void criarInimigos(int quantidade) {
        int pontos = new Integer(pai.jLabelPontos.getText()).intValue();
        // se a pontuação for múltiplo de 6, a velocidade do inimigo aumenta
        if ((pontos > 0) && (pontos % 6 == 0)) {
            velocidadeInimigo++;
            quantidadeDeTirosInimigo++;
        }
        // se a pontuação chegar a 50, o chefão é criado
        if ((pontos > 0) && (pontos % 50 == 0) && (ativarChefao != true)) {
            ativarChefao = true;
            chefoes.add(new Chefao("/imagens/chefao.png", this, pai.jPanelTela.getWidth() - 195, pai.jPanelTela.getHeight() / 4, velocidadeInimigo));
            pai.jProgressBarVidaChefao.setValue(500);
        }

        // cria novos inimigos
        for (int i = 0; i < quantidade; i++) {
            inimigos.add(new Inimigo("/imagens/joaninha.png", this, pai.jPanelTela.getWidth(), posicaoVertical(), velocidadeInimigo));
        }
    }

    /**
     * Verifica se houve colisoes entre os objetos na tela.
     */
    private void verificaColisoes() {
        // descobrindo as dimensoes e posicao dos jogadores
        Rectangle rJogadorPrincipal = personagemPrincipal.getLimites();
        // verificando colisão com powerups velocidade
        if (powerupVelocidade == true) {
            ArrayList<Velocidade> removerVelocidade = new ArrayList<Velocidade>();
            for (Velocidade v : velocidade) {
                Rectangle rVelocidade = v.getLimites();
                if (rVelocidade.intersects(rJogadorPrincipal)) {
                    if (pai.jProgressBarVelocidade.getValue() < 100) {
                        personagemPrincipal.incrementarVelocidadePersonagem(1);
                        pai.jProgressBarVelocidade.setValue(pai.jProgressBarVelocidade.getValue() + 25);
                        removerVelocidade.add(v);
                    }
                    removerVelocidade.add(v);
                }
            }
            // removendo powerup após colisão
            for (Velocidade v : removerVelocidade) {
                velocidade.remove(v);
                if (velocidade.size() == 0) {
                    powerupVelocidade = false;
                }
            }
        }
        // verificando colisão powerup potência do canhão
        if (powerupPotenciaCanhao == true) {
            ArrayList<PotenciaCanhaoDePlasma> removerPotencia = new ArrayList<PotenciaCanhaoDePlasma>();
            for (PotenciaCanhaoDePlasma p : potenciacanhao) {
                Rectangle rPotencia = p.getLimites();
                if (rPotencia.intersects(rJogadorPrincipal)) {
                    if (pai.jProgressBarPotenciaCanhao.getValue() < 100) {
                        personagemPrincipal.incrementarVelocidadeTiro(1);
                        personagemPrincipal.incrementarNumeroDeDisparos(1);
                        personagemPrincipal.incrementarNumeroDeDisparosDiagonalSuperior(1);
                        personagemPrincipal.incrementarNumeroDeDisparosDiagonalInferior(2);
                        personagemPrincipal.incrementarPotenciaCanhao(1);
                        pai.jProgressBarPotenciaCanhao.setValue(pai.jProgressBarPotenciaCanhao.getValue() + 25);
                        removerPotencia.add(p);
                    }
                    removerPotencia.add(p);
                }
            }
            // removendo powerup após colisão
            for (PotenciaCanhaoDePlasma p : removerPotencia) {
                potenciacanhao.remove(p);
                if (potenciacanhao.size() == 0) {
                    powerupPotenciaCanhao = false;
                }
            }
        }
        // descobrindo as dimensoes e posicao do chefao
        if (ativarChefao == true) {
            ArrayList<CanhaoDePlasma> tiroChefao1 = new ArrayList<CanhaoDePlasma>();
            ArrayList<CanhaoDePlasma> tiroChefao2 = new ArrayList<CanhaoDePlasma>();
            ArrayList<CanhaoDePlasma> tiroChefao3 = new ArrayList<CanhaoDePlasma>();
            // verificando a colisão do tiro do chefão com o personagem
            for (Chefao tirochefao : chefoes) {
                for (CanhaoDePlasma cC : tirochefao.canhaoChefao1) {
                    Rectangle rcC = new Rectangle(cC.getLimites());
                    if (rcC.intersects(rJogadorPrincipal)) {
                        // tratando a colisão
                        colisaoCanhaoChefaoXJogador();
                        // adicionando tiro para ser removido
                        tiroChefao1.add(cC);
                    }
                }
                // removendo disparos atingidos
                for (CanhaoDePlasma ci : tiroChefao1) {
                    tirochefao.canhaoChefao1.remove(ci);
                }
                for (CanhaoDePlasma cC : tirochefao.canhaoChefao2) {
                    Rectangle rcC = new Rectangle(cC.getLimites());
                    if (rcC.intersects(rJogadorPrincipal)) {
                        // tratando a colisão
                        colisaoCanhaoChefaoXJogador();
                        // adicionando tiro para ser removido
                        tiroChefao2.add(cC);
                    }
                }
                // removendo disparos atingidos
                for (CanhaoDePlasma ci : tiroChefao2) {
                    tirochefao.canhaoChefao2.remove(ci);
                }
                for (CanhaoDePlasma cC : tirochefao.canhaoChefao3) {
                    Rectangle rcC = new Rectangle(cC.getLimites());
                    if (rcC.intersects(rJogadorPrincipal)) {
                        // tratando a colisão
                        colisaoCanhaoChefaoXJogador();
                        // adicionando tiro para ser removido
                        tiroChefao3.add(cC);
                    }
                }
                // removendo disparos atingidos
                for (CanhaoDePlasma ci : tiroChefao3) {
                    tirochefao.canhaoChefao3.remove(ci);
                }

            }
            // verficando a colisão do chefão com o personagem
            for (Chefao colisaoChefao : chefoes) {
                Rectangle rChefao = new Rectangle(colisaoChefao.getLimites());
                if (rChefao.intersects(rJogadorPrincipal)) {
                    // tratando a colisão
                    colisaoJogadorXChefao();
                }
            }
        }
        ArrayList<Inimigo> colidiram = new ArrayList<Inimigo>();
        // verificando a colisão do personagem com o inimigo
        for (Inimigo ini : inimigos) {
            Rectangle rInimigo = new Rectangle(ini.getLimites());
            // usufruindo do método intersects da classe Rectangle
            if (rJogadorPrincipal.intersects(rInimigo)) {
                // tratando a colisão
                colisaoJogadorXInimigo();
                int pontos = new Integer(pai.jLabelPontos.getText()).intValue();
                if (pontos % 6 == 0) {
                    boolean powerup = r.nextBoolean();
                    if (powerup == true) {
                        if (pai.jProgressBarVelocidade.getValue() < 100) {
                            velocidade.add(new Velocidade("/imagens/velocidade.png", this, ini.getX(), ini.getY()));
                            powerupVelocidade = true;
                        } else if (pai.jProgressBarPotenciaCanhao.getValue() < 100) {
                            potenciacanhao.add(new PotenciaCanhaoDePlasma("/imagens/potencia.png", this, ini.getX(), ini.getY()));
                            powerupPotenciaCanhao = true;
                        }
                    }
                    if (powerup == false) {
                        if (pai.jProgressBarPotenciaCanhao.getValue() < 100) {
                            potenciacanhao.add(new PotenciaCanhaoDePlasma("/imagens/potencia.png", this, ini.getX(), ini.getY()));
                            powerupPotenciaCanhao = true;
                        } else if (pai.jProgressBarVelocidade.getValue() < 100) {
                            velocidade.add(new Velocidade("/imagens/velocidade.png", this, ini.getX(), ini.getY()));
                            powerupVelocidade = true;
                        }
                    }
                }
                colidiram.add(ini);
            }
        }
        // removendo inimigos que sofreram colisão
        for (Inimigo i : colidiram) {
            inimigos.remove(i);
            // criando novos inimigos
            int totalInimigos = 5 - inimigos.size();
            // para garantir que pelo menos 1 novo inimigo seja criado
            if (totalInimigos <= 1) {
                criarInimigos(1);
            } else {
                totalInimigos = r.nextInt(totalInimigos + 1);
                /*if (totalInimigos == 0) {
                totalInimigos = 1;
                }*/
                criarInimigos(totalInimigos);
            }
        }
        // verificando colisao do tiro inimigo com o personagem principal
        ArrayList<CanhaoDePlasma> tiroInimigo = new ArrayList<CanhaoDePlasma>();
        for (Inimigo ini : inimigos) {
            for (CanhaoDePlasma cI : ini.canhaoInimigo) {
                // descobrindo as dimensões do tiro inimigo
                Rectangle rcI = new Rectangle(cI.getLimites());
                if (rcI.intersects(rJogadorPrincipal)) {
                    // tratando a colisão
                    colisaoCanhaoInimigoXJogador();
                    // adicionando tiro para ser removido
                    tiroInimigo.add(cI);
                }
            }
            // removendo disparos atingidos
            for (CanhaoDePlasma ci : tiroInimigo) {
                ini.canhaoInimigo.remove(ci);
            }
        }
        ArrayList<CanhaoDePlasma> tiroDescartado = new ArrayList<CanhaoDePlasma>();
        ArrayList<CanhaoDePlasma> tiroDescartado1 = new ArrayList<CanhaoDePlasma>();
        ArrayList<CanhaoDePlasma> tiroDescartado2 = new ArrayList<CanhaoDePlasma>();
        ArrayList<Inimigo> inimigo = new ArrayList<Inimigo>();
        ArrayList<Chefao> chefao = new ArrayList<Chefao>();
        // verificando colisão do tiro do personagem principal no inimigo
        for (CanhaoDePlasma cJ : personagemPrincipal.obterCanhao()) {
            Rectangle rCj = new Rectangle(cJ.getLimites());
            for (Inimigo ini : inimigos) {
                Rectangle rInimigo = new Rectangle(ini.getLimites());
                // verificando se o disparo atingiu inimigo
                if (rCj.intersects(rInimigo)) {
                    // adicionando disparo no vetor para ser removido
                    tiroDescartado.add(cJ);
                    // tratando o dano do tiro no personagem
                    ini.vidaInimigo = ini.obterVidaInimigo() - cJ.obterDanoCanhaoDePlasma();
                    // vetor para verificar inimigos que sofreram tiros
                    inimigo.add(ini);
                }
            }
            // verificando se o disparo atingiu o chefão
            for (Chefao chef : chefoes) {
                Rectangle rChefao = new Rectangle(chef.getLimites());
                if (rChefao.intersects(rCj)) {
                    // adicionando disparo no vetor para ser removido
                    tiroDescartado.add(cJ);
                    // tratando o dano do tiro no chefão
                    chef.vidaChefao = chef.obterVidaChefao() - cJ.obterDanoCanhaoDePlasma();
                    pai.jProgressBarVidaChefao.setValue(chef.vidaChefao);
                    chefao.add(chef);
                }
            }
        }
        // verificando colisão do tiro do personagem principal no inimigo
        for (CanhaoDePlasma cJ : personagemPrincipal.obterCanhao1()) {
            Rectangle rCj = new Rectangle(cJ.getLimites());
            for (Inimigo ini : inimigos) {
                Rectangle rInimigo = new Rectangle(ini.getLimites());
                // verificando se o disparo atingiu inimigo
                if (rCj.intersects(rInimigo)) {
                    // adicionando disparo no vetor para ser removido
                    tiroDescartado1.add(cJ);
                    // tratando o dano do tiro no personagem
                    ini.vidaInimigo = ini.obterVidaInimigo() - cJ.obterDanoCanhaoDePlasma();
                    // vetor para verificar inimigos que sofreram tiros
                    inimigo.add(ini);
                }
            }
            // verificando se o disparo atingiu o chefão
            for (Chefao chef : chefoes) {
                Rectangle rChefao = new Rectangle(chef.getLimites());
                if (rChefao.intersects(rCj)) {
                    // adicionando disparo no vetor para ser removido
                    tiroDescartado1.add(cJ);
                    // tratando o dano do tiro no chefão
                    chef.vidaChefao = chef.obterVidaChefao() - cJ.obterDanoCanhaoDePlasma();
                    pai.jProgressBarVidaChefao.setValue(chef.vidaChefao);
                    chefao.add(chef);

                }
            }
        }
        // verificando colisão do tiro do personagem principal no inimigo
        for (CanhaoDePlasma cJ : personagemPrincipal.obterCanhao2()) {
            Rectangle rCj = new Rectangle(cJ.getLimites());
            for (Inimigo ini : inimigos) {
                Rectangle rInimigo = new Rectangle(ini.getLimites());
                // verificando se o disparo atingiu inimigo
                if (rCj.intersects(rInimigo)) {
                    // adicionando disparo no vetor para ser removido
                    tiroDescartado2.add(cJ);
                    // tratando o dano do tiro no personagem
                    ini.vidaInimigo = ini.obterVidaInimigo() - cJ.obterDanoCanhaoDePlasma();
                    // vetor para verificar inimigos que sofreram tiros
                    inimigo.add(ini);
                }
            }
            // verificando se o disparo atingiu o chefão
            for (Chefao chef : chefoes) {
                Rectangle rChefao = new Rectangle(chef.getLimites());
                if (rChefao.intersects(rCj)) {
                    // adicionando disparo no vetor para ser removido
                    tiroDescartado2.add(cJ);
                    // tratando o dano do tiro no chefão
                    chef.vidaChefao = chef.obterVidaChefao() - cJ.obterDanoCanhaoDePlasma();
                    pai.jProgressBarVidaChefao.setValue(chef.vidaChefao);
                    chefao.add(chef);

                }
            }
        }
        // removendo tiros colididos com inimigo
        for (CanhaoDePlasma c : tiroDescartado) {
            personagemPrincipal.vetorCanhaoDePlasma.remove(c);
        }
        // removendo tiros colididos com inimigo
        for (CanhaoDePlasma c1 : tiroDescartado1) {
            personagemPrincipal.vetorCanhaoDePlasma1.remove(c1);
        }
        // removendo tiros colididos com inimigo
        for (CanhaoDePlasma c2 : tiroDescartado2) {
            personagemPrincipal.vetorCanhaoDePlasma2.remove(c2);
        }
        // removendo inimigos mortos
        for (Inimigo ini : inimigo) {
            // verificando se a vida do inimigo é <= 0
            if (ini.vidaInimigo <= 0) {
                // aumentando a pontuação
                int pontos = new Integer(pai.jLabelPontos.getText()).intValue();
                pontos++;
                pai.jLabelPontos.setText("" + pontos);
                // aumentando o poder
                pai.jProgressBarPoder.setValue(pai.jProgressBarPoder.getValue() + 5);
                if (pontos % 6 == 0) {
                    boolean powerup = r.nextBoolean();
                    if (powerup == true) {
                        if (pai.jProgressBarVelocidade.getValue() < 100) {
                            velocidade.add(new Velocidade("/imagens/velocidade.png", this, ini.getX(), ini.getY()));
                            powerupVelocidade = true;
                        } else if (pai.jProgressBarPotenciaCanhao.getValue() < 100) {
                            potenciacanhao.add(new PotenciaCanhaoDePlasma("/imagens/potencia.png", this, ini.getX(), ini.getY()));
                            powerupPotenciaCanhao = true;
                        }
                    }
                    if (powerup == false) {
                        if (pai.jProgressBarPotenciaCanhao.getValue() < 100) {
                            potenciacanhao.add(new PotenciaCanhaoDePlasma("/imagens/potencia.png", this, ini.getX(), ini.getY()));
                            powerupPotenciaCanhao = true;
                        } else if (pai.jProgressBarVelocidade.getValue() < 100) {
                            velocidade.add(new Velocidade("/imagens/velocidade.png", this, ini.getX(), ini.getY()));
                            powerupVelocidade = true;
                        }
                    }
                }
                // removendo inimigos mortos
                inimigos.remove(ini);
                // criando novos inimigos
                int totalInimigos = 5 - inimigos.size();
                // para garantir que pelo menos 1 novo inimigo seja criado
                if (totalInimigos <= 1) {
                    totalInimigos = 1;
                    criarInimigos(totalInimigos);
                } else {
                    totalInimigos = r.nextInt(totalInimigos) + 1;
                    /* if (totalInimigos == 0) {
                    totalInimigos = 1;
                    }*/
                    criarInimigos(totalInimigos);
                }
            }
        }
        // vericando se o chefão morreu
        for (Chefao chef : chefao) {
            if (chef.vidaChefao <= 0) {
                pai.jProgressBarVidaChefao.setValue(0);
                chefoes.remove(chef);
                // aumentando a pontuação ao matar o chefão
                int pontos = new Integer(pai.jLabelPontos.getText()).intValue();
                pontos += 10;
                pai.jLabelPontos.setText("" + pontos);
                // aumentando o poder ao matar o chefão
                pai.jProgressBarPoder.setValue(pai.jProgressBarPoder.getValue() + 25);
                ativarChefao = false;
                pai.jLabeAtivarChefao.setText("off");
            }
        }
    }

    /**
     * Trata a colisão do jogador principal com o inimigo
     */
    public void colisaoJogadorXInimigo() {
        int pontos = new Integer(pai.jLabelPontos.getText()).intValue();
        pontos++;
        // se o escudo de força estiver desativado, sofrerá dano
        if (personagemPrincipal.obterInvencibilidade() == false) {
            // ao ser tocado por um inimigo o personagem perde 25 pontos de vida
            pai.jProgressBarVida.setValue(pai.jProgressBarVida.getValue() - 25);
            // cada inimigo morto adiciona 1 ponto para o jogador
            pai.jLabelPontos.setText("" + pontos);
            // se a vida do personagem principal zerar, o jogo acaba
            if (pai.jProgressBarVida.getValue() <= 0) {
                pai.jProgressBarVida.setValue(0);
                jogoAtivo = false;
            }
        } else {
            pai.jLabelPontos.setText("" + pontos);
            // cada inimigo morto aumenta 5 pontos de poder
            pai.jProgressBarPoder.setValue(pai.jProgressBarPoder.getValue() + 5);
        }
    }

    /**
     * Trata a colisão do jogador principal com o chefão
     */
    public void colisaoJogadorXChefao() {
        // se o escudo de força estiver desativado, sofrerá dano
        if (personagemPrincipal.obterInvencibilidade() != true) {
            int vida = pai.jProgressBarVida.getValue();
            vida -=
                    50;
            // se a vida do personagem principal zerar, o jogo acaba
            if (vida <= 0) {
                pai.jProgressBarVida.setValue(0);
                jogoAtivo = false;
            } else {
                pai.jProgressBarVida.setValue(vida);
            }

        }
    }

    /**
     * Trata a colisão do tiro do chefão com o jogador
     */
    public void colisaoCanhaoChefaoXJogador() {
        // se o escudo de força estiver desativado, sofrerá dano
        if (personagemPrincipal.obterInvencibilidade() == false) {
            // cada disparo diminui 20 pontos da vida do jogador
            int vida = pai.jProgressBarVida.getValue();
            vida -=
                    20;
            // se a vida do personagem principal zerar, o jogo acaba
            if (vida <= 0) {
                pai.jProgressBarVida.setValue(0);
                jogoAtivo = false;
            } else {
                pai.jProgressBarVida.setValue(vida);
            }

        }
    }

    /**
     *  Trata a colisão do tiro do inimigo com o jogador
     */
    public void colisaoCanhaoInimigoXJogador() {
        if (personagemPrincipal.obterInvencibilidade() == false) {
            int vida = pai.jProgressBarVida.getValue();
            // cada disparo diminui 10 pontos da vida do jogador
            vida -=
                    10;
            // se a vida do personagem principal zerar, o jogo acaba
            if (vida <= 0) {
                pai.jProgressBarVida.setValue(0);
                jogoAtivo = false;
            } else {
                pai.jProgressBarVida.setValue(vida);
            }

        }
    }

    /**
     * Gera a posição vertical de cada inimigo aleatoriamente
     * @return
     */
    public int posicaoVertical() {
        int vertical = r.nextInt(pai.jPanelTela.getHeight());
        // se estiver fora dos limites do eixo Y, assume nova posição
        if (vertical > 264) {
            vertical = 264;
        }
        return vertical;
    }

    /**
     * Classe interna e privada para tratar os eventos de tecla pressionada
     * e tecla solta.
     * Neste exemplo, sempre que uma tecla for pressionada, o j1 irá se mover
     *
     * Uma classe interna consegue acessar todos os atributos (mesmo os privados)
     * da classe mais externa que a contém.
     *
     */
    private class TratadorEventoTeclado extends KeyAdapter {

        /**
         * Quando ocorrer o evento de tecla solta, será invocado o método teclaSolta
         * do objeto j1
         * @param e evento do teclado
         */
        @Override
        public void keyReleased(KeyEvent e) {
            if (personagemPrincipal != null) {
                personagemPrincipal.teclaSolta(e);
            }
        }

        /**
         * Quando ocorrer o evento de tecla pressionada, será invocado o
         * método teclaPressionada do objeto j1
         * @param e evento do teclado
         */
        @Override
        public void keyPressed(KeyEvent e) {
            if (personagemPrincipal != null) {
                personagemPrincipal.teclaPressionada(e);
            }
        }
    }
}
